#include  <All IP Procedures>

// version 2013_03_11

Menu "Dirac"
"measureAngle"
"StoreCentrePosition"
"PutCursorInCentre"
"prepareCursorsForDiracWF"
"extractDiracWF"
"ExtractFitResults"
"DiracPointfromX"
" findFermiVelocityGammaK"
"DiracPointfromAvg"
End

Proc StoreCentrePosition()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	variable pA=pcsr(A)
	variable qA=qcsr(A)
	Variable/G diracPospA=pA
	Variable/G diracPosqA=qA
End

Proc PutCursorInCentre()
	DRC_csr2Centre()
End

Proc MeasureAngle()
	DRC_measureAngle()
End

Proc PrepareCursorsForDiracWF()
	DRC_putCursorsForDiracWF()
End

Proc ExtractDiracWF()
	PROI_extract_WF()
End

Proc ExtractFitResults()
	FIT_extract_results()
End

Proc DiracPointfromX()
	 DRC_DiracPointfromX()
End

Proc findFermiVelocityGammaK()
	DRC_FermiVelocityGammaKappa()
End

Proc DiracPointFromAvg()
	 DRC_DiracPointfromAvg()
End

Function DRC_csr2Centre()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	NVAR diracPospA=root:$(dfStr):diracPospA
	NVAR diracPosqA=root:$(dfStr):diracPosqA
	String stackStr=SIS_getImageStackName()	
	Cursor/P/I A $stackStr,diracPospA,diracPosqA
End

Function DRC_measureAngle()
	String ctrlname
	NVAR rotangle=root:SIS:rotangle
	SetDataFolder root:SIS
	Variable/G kcalib=0.0031
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	NVAR kcalib=root:SIS:kcalib
	//
	variable suggestedAngle
	if (strlen(csrInfo(A))>0 & strlen(csrInfo(B))>0)
		if (stringMatch(num2Str(pcsr(A)),"NaN")!=1 && stringMatch(num2Str(pcsr(B)),"NaN")!=1)
			variable p1=pcsr(A)
			variable p2=pcsr(B)
			variable q1=qcsr(A)
			variable q2=qcsr(B)
			suggestedAngle=round(atan(abs(q2-q1)/abs(p2-p1))*1800/Pi)/10
		endif
	else
		Print "put cursors on screen please"
	endif
	rotangle = suggestedAngle
	print suggestedAngle
end

Function DRC_putCursorsForDiracWF()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	String winNameStr=SIS_getWinName()
	// input
	NVAR rotangle=root:SIS:rotangle
	variable halfWidth=100
	prompt halfWidth, "Distance from csrA"
	variable angle=rotangle
	prompt angle, "angle fomr horizontal"
	variable rotation=1
	Prompt rotation, "Rotate 90 deg", popup "no; yes"
	doPrompt "input put cursors for Dirac", halfWidth, angle, rotation
	if (V_flag ==0)
		if (stringMatch(num2Str(pcsr(A)),"NaN")!=1)
			variable p1=pcsr(A)
			variable q1=qcsr(A)
			if (rotation == 1)
				variable newp1=p1+round(halfWidth*cos(angle*Pi/180))
				variable newq1=q1-round(halfWidth*sin(angle*Pi/180))
				variable newp2=p1-round(halfWidth*cos(angle*Pi/180))
				variable newq2=q1+round(halfWidth*sin(angle*Pi/180))
				Cursor/P/I B $stackStr,newp2,newq2
				Cursor/P/I A$stackStr,newp1,newq1
			else
				variable newp1r=p1+round(halfWidth*cos((angle+90)*Pi/180))
				variable newq1r=q1-round(halfWidth*sin((angle+90)*Pi/180))
				variable newp2r=p1-round(halfWidth*cos((angle+90)*Pi/180))
				variable newq2r=q1+round(halfWidth*sin((angle+90)*Pi/180))
				Cursor/P/I B $stackStr,newp2r,newq2r
				Cursor/P/I A$stackStr,newp1r,newq1r
			endif
		endif
	endif
end

Function DRC_DiracPointfromX()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// input
	NVAR kcalib=root:SIS:kcalib
	String y1Str
	Prompt y1Str, "enter first wave",popup WaveList("F*_9",";","")
	String y2Str
	Prompt y2Str, "enter second wave",popup WaveList("F*_14",";","")
	String xStr
	Prompt xStr, "enter K wave",popup WaveList("*_Q",";","")
	variable errormode=2
	Prompt errorMode, "Error Mode", popup "auto; input errors"
	variable intercepterror=1
	Prompt intercepterror, "Consider intercept error", popup "no; yes"
	variable calib=kcalib
	Prompt calib,"k-space calibration"
	doprompt "calculate Dirac energy" y1Str,y2Str,xStr,errorMode,intercepterror,calib
	if (V_Flag==0)
		kcalib=calib
		make/n=2/o W_coef,W_sigma,W_ParamConfidenceInterval
		wave W_coef,W_sigma,W_ParamConfidenceInterval
		// auto-estimate errors
		CurveFit/NTHR=0/ODR=1/Q/G line  $y2Str[pcsr(A),pcsr(B)] /X=$xStr /D 
		variable a0_2=W_coef[0]
		variable a1_2=W_coef[1]
		duplicate/o $y2Str diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0_2+a1_2*xx[p])
		Wavestats/R=(pcsr(A),pcsr(B))/Q diff
		diff=V_rms
		// calculate errors from fit
		String errStr2="E"+y2str[1,strlen(y2Str)]
		wave we2 = $errStr2
		if (errorMode == 2)
			diff = we2
		endif
		//fit
		CurveFit/NTHR=0/Q line  $y2Str[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D /F={0.683000, 4}
		// results
		a0_2=W_coef[0]
		a1_2=W_coef[1]
		// error
		variable sig0_2=W_ParamConfidenceInterval[0]
		variable sig1_2=W_ParamConfidenceInterval[1]
		// plot line
		String fit2Str="fit_"+y2Str
		wavestats/Q $xStr
		variable xl=V_min
		variable xr=V_max
		SetScale/I x xl,xr,"",$fit2Str
		wave wf2=$fit2Str
		wf2=a0_2+a1_2*x
		// auto estimate errors	
		CurveFit/NTHR=0/ODR=1/Q/G line  $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D 
		variable a0_1=W_coef[0]
		variable a1_1=W_coef[1]
		duplicate/o $y1Str diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0_1+a1_1*xx[p])
		Wavestats/R=(pcsr(A),pcsr(B))/Q diff
		diff=V_rms 
		// calculate errors
		String errStr1="E"+y1str[1,strlen(y1Str)]
		wave we1 = $errStr1
		if (errorMode == 2)
			diff = we1
		endif
		// fit
		CurveFit/NTHR=0/Q line $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D/F={0.683000, 4}
		// results
		a0_1=W_coef[0]
		a1_1=W_coef[1]
		// errors
		variable sig0_1=W_ParamConfidenceInterval[0]
		variable sig1_1=W_ParamConfidenceInterval[1]
		// plot
		String fit1Str="fit_"+y1Str
		SetScale/I x xl,xr,"",$fit1Str
		wave wf1=$fit1Str
		wf1=a0_1+a1_1*x
		// print results
		variable E_d=(a0_2-a0_1)/(a1_1-a1_2)
		variable hplanck=6.626007E-34
		variable eVJ=1.6021765E-19
		variable avgslope=(abs(a1_1)+abs(a1_2))*0.5
		variable deltaavgslope=0.5*(sig1_2+sig1_1)
		variable VF=1/(hplanck*avgslope*calib*0.5*1E10/eVJ) //0.0028 for Ir001
		variable deltaVF=1/(hplanck*calib*0.5*1E10/eVJ)*deltaavgslope/(avgslope)^2 
		// print for debug
		print a0_1,a0_2,a1_1,a1_2
		print sig0_1,sig0_2, sig1_1,sig1_2
		// error on E_D
		variable err_E_d=sqrt( (sig0_2^2+sig0_1^2) /  (abs(a1_1-a1_2))^2 + (intercepterror-1) * (sig1_2^2+sig1_1^2) * ((a0_2-a0_1)/(a1_1-a1_2)^2)^2 )
		string  msgStr="The Dirac point is located at: "+num2Str(E_d)+" pm "+num2Str(err_E_d)+"\r Fermi velocity is: "+num2Str(VF)+" pm "+num2str(deltaVF)
		make/N=2/O edw
		edw=E_d
		String wfpStr="WF"+dfStr+"_P"
		variable leftxWF=leftx($wfpStr)
		variable rightxWF=rightx($wfpStr)
		SetScale/I x leftxWF,rightxWF,"",edw
		TextBox/C/N=text1/A=RB msgStr
		print VF
		// append on waterfall
		String waterfallStr = "WF"+dfStr
		DRC_lineToWaterfallVert(waterfallStr,fit2Str)
		DRC_lineToWaterfallVert(waterfallStr,fit1Str)
		DRC_lineToWaterfallBottom(waterfallStr,"edw")
		ModifyGraph rgb=(65280,0,0)
		TextBox/C/N=text1/A=RB msgStr
	endif
end

Function DRC_FermiVelocityGammaKappa()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// input
	NVAR kcalib=root:SIS:kcalib
	String y1Str
	Prompt y1Str, "enter first wave wave",popup WaveList("F*_9",";","")
	String xStr
	Prompt xStr, "enter first wave wave",popup WaveList("*_Q",";","")
	variable errormode=2
	Prompt errorMode, "Error Mode", popup "auto; input errors"
	variable intercepterror=1
	Prompt intercepterror, "Consider intercept error", popup "no; yes"
	variable calib=kcalib
	Prompt calib,"k-space calibration"
	doprompt "calculate Dirac energy" y1Str,xStr,errorMode,intercepterror,calib
	if (V_Flag==0)
		kcalib=calib
		make/n=2/o W_coef,W_sigma,W_ParamConfidenceInterval
		wave W_coef,W_sigma,W_ParamConfidenceInterval
		// auto-estimate errors
		CurveFit/NTHR=0/ODR=1/Q/G line  $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D 
		variable a0_1=W_coef[0]
		variable a1_1=W_coef[1]
		duplicate/o $y1Str diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0_1+a1_1*xx[p])
		Wavestats/R=(pcsr(A),pcsr(B))/Q diff
		diff=V_rms
		// calculate errors from fit
		String errStr1="E"+y1str[1,strlen(y1Str)]
		wave we1 = $errStr1
		if (errorMode == 2)
			diff = we1
		endif
		//fit
		CurveFit/NTHR=0/Q line  $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D /F={0.683000, 4}
		// results
		a0_1=W_coef[0]
		a1_1=W_coef[1]
		// error
		variable sig0_1=W_ParamConfidenceInterval[0]
		variable sig1_1=W_ParamConfidenceInterval[1]
		// plot line
		String fit1Str="fit_"+y1Str
		wavestats/Q $xStr
		variable xl=V_min
		variable xr=V_max
		SetScale/I x xl,xr,"",$fit1Str
		wave wf1=$fit1Str
		wf1=a0_1+a1_1*x
		// calculate ED
		// 100 = a02 +a12*x
		variable E_d = (100-a0_1)/a1_1
		variable err_E_d = abs(sig0_1/a1_1)+abs((100-a0_1)*sig1_1/(a1_1^2))
		// calculatre VF
		variable hplanck=6.626007E-34
		variable eVJ=1.6021765E-19
		variable slope=abs(a1_1)
		variable deltaslope=sig1_1
		variable VF=1/(hplanck*slope*calib*0.5*1E10/eVJ) 
		variable deltaVF=1/(hplanck*calib*0.5*1E10/eVJ)*deltaslope/(slope)^2 
		string  msgStr="The Dirac point is located at: "+num2Str(E_d)+" pm "+num2Str(err_E_d)+"\r Fermi velocity is: "+num2Str(VF)+" pm "+num2str(deltaVF)
		make/N=2/O edw
		edw=E_d
		String wfpStr="WF"+dfStr+"_P"
		variable leftxWF=leftx($wfpStr)
		variable rightxWF=rightx($wfpStr)
		SetScale/I x leftxWF,rightxWF,"",edw
		TextBox/C/N=text2/A=RB msgStr
		print VF
		// append on waterfall
		String waterfallStr = "WF"+dfStr
		DRC_lineToWaterfallVert(waterfallStr,fit1Str)
		DRC_lineToWaterfallBottom(waterfallStr,"edw")
		ModifyGraph rgb=(65280,0,0)
	endif
end

Function DRC_lineToWaterfallVert(graphNameStr,waveNameStr)
	String graphNameStr
	String waveNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/VERT $waveNameStr 
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/VERT $waveNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function DRC_lineToWaterfallBottom(graphNameStr,waveNameStr)
	String graphNameStr
	String waveNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph $waveNameStr 
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph $waveNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function DRC_DiracPointFromAvg()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// input
	NVAR kcalib=root:SIS:kcalib
	String y1Str
	Prompt y1Str, "enter first wave wave",popup WaveList("*_9",";","")
	String y2Str
	Prompt y2Str, "enter first wave wave",popup WaveList("*_14",";","")
	String xStr
	Prompt xStr, "enter first wave wave",popup WaveList("*_Q",";","")
	variable errormode=2
	Prompt errorMode, "Error Mode", popup "auto; fit errors"
	variable intercepterror=1
	Prompt intercepterror, "Consider intercept error", popup "no; yes"
	variable calib=kcalib
	Prompt calib,"k-space calibration"
	doprompt "calculate Dirac energy" y1Str,y2Str,xStr,errorMode,intercepterror,calib
	if (V_Flag==0)
		kcalib=calib
		make/n=2/o W_coef,W_sigma,W_ParamConfidenceInterval
		wave W_coef,W_sigma,W_ParamConfidenceInterval
		String yplusStr="FR"+dfStr+"_PL"
		String yminusStr="FR"+dfStr+"_MN"
		String plusErrStr="ER"+dfstr+"_PL"
		String minusErrStr="ER"+dfstr+"_MN"
		String e1Str="E"+y1str[1,strlen(y1Str)]
		String e2Str="E"+y2str[1,strlen(y2Str)]
		variable pa=pcsr(A)
		variable pb=pcsr(B)
		duplicate/o $y1Str wave3
		wave wave1=$y1Str
		wave wave2=$y2Str
		wave ewave1=$e1Str
		wave ewave2=$e2Str
		duplicate/o wave1 $yplusStr
		duplicate/o wave1 $yminusStr
		duplicate/o wave1 $minusErrStr
		duplicate/o wave1 $plusErrStr
		wave pluswave=$yplusStr
		wave minuswave=$yminusStr
		wave wep= $plusErrStr
		wave wem=$minusErrStr
		pluswave = 0.5*abs(wave1 + wave2)
		minuswave = wave2
		// process minuswave
		// estimate errors
		CurveFit/NTHR=0/ODR=1/Q/G line  minuswave[pa,pb] /X=$xStr /D 
		variable a0=W_coef[0]
		variable a1=W_coef[1]
		duplicate/o minuswave diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0+a1*xx[p])
		Wavestats/R=(pa,pb)/Q diff
		diff=V_sdev
		// calculate errors
	 	wem = ewave2
		if (errorMode == 2)
			diff = wem
		endif
		String graphNameStr=y1str[0,strlen(y1Str)-3]
		SIS_graphToFrontRight(graphNameStr,yminusStr,xStr,"left5")
		ErrorBars $yminusStr Y,wave=($minusErrStr,$minusErrStr)
		//fit
		CurveFit/NTHR=0/Q line  $yminusStr[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D /F={0.683000, 4}
		// results
		variable a0_2=W_coef[0]
		variable a1_2=W_coef[1]
		// error
		variable sig0_2=W_ParamConfidenceInterval[0]
		variable sig1_2=W_ParamConfidenceInterval[1]
		// plot line
		String fit2Str="fit_"+y2Str
		wavestats/Q $xStr
		variable xl=V_min
		variable xr=V_max
		SetScale/I x xl,xr,"",$fit2Str
		wave wf2=$fit2Str
		wf2=a0_2+a1_2*x
		// process pluswave
		SIS_graphToFrontRight(graphNameStr,yplusStr,xStr,"left5")
		CurveFit/NTHR=0/ODR=1/Q/G line  pluswave[pa,pb] /X=$xStr /D 
		a0=W_coef[0]
		a1=W_coef[1]
		duplicate/o minuswave diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0+a1*xx[p])
		Wavestats/R=(pa,pb)/Q diff
		diff=V_sdev
		// calculate errors
		wep=0.5*(ewave1+ewave2)
		if (errorMode == 2)
			diff = 0.5*(wep)
		endif
		SIS_graphToFrontRight(graphNameStr,yplusStr,xStr,"left5")
		ErrorBars $yplusStr Y,wave=($plusErrStr,$plusErrStr)
		// fit
		CurveFit/NTHR=0/Q line $yplusStr[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D/F={0.683000, 4}
		// results
		variable a0_1=W_coef[0]
		variable a1_1=W_coef[1]
		// errors
		variable sig0_1=W_ParamConfidenceInterval[0]
		variable sig1_1=W_ParamConfidenceInterval[1]
		// plot
		String fit1Str="fit_"+yplusStr
		SetScale/I x xl,xr,"",$fit1Str
		wave wf1=$fit1Str
		wf1=a0_1+a1_1*x		
		// print results
		variable E_d=(a0_2-a0_1)/(a1_1-a1_2)
		variable hplanck=6.626007E-34
		variable eVJ=1.6021765E-19
		variable avgslope=abs(a1_1-a1_2)
		variable deltaavgslope=(sig1_1+sig1_2)
		variable VF=1/(hplanck*avgslope*calib*0.5*1E10/eVJ) //0.0028 for Ir001
		variable deltaVF=1/(hplanck*calib*0.5*1E10/eVJ)*deltaavgslope/(avgslope)^2 
		// print for debug
		print a0_1,a0_2,a1_1,a1_2
		print sig0_1,sig0_2, sig1_1,sig1_2
		// error on E_D
		variable err_E_d=sqrt( (sig0_2^2+sig0_1^2) /  (abs(a1_1-a1_2))^2 + (intercepterror-1) * (sig1_2^2+sig1_1^2) * ((a0_2-a0_1)/(a1_1-a1_2)^2)^2 )
		string  msgStr="The Dirac point is located at: "+num2Str(E_d)+" pm "+num2Str(err_E_d)+"\r Fermi velocity is: "+num2Str(VF)+" pm "+num2str(deltaVF)
		make/N=2/O edw
		edw=E_d
		String wfpStr="WF"+dfStr+"_P"
		variable leftxWF=leftx($wfpStr)
		variable rightxWF=rightx($wfpStr)
		SetScale/I x leftxWF,rightxWF,"",edw
		TextBox/C/N=text1/A=RB msgStr
		print VF
		// append on waterfall
		String waterfallStr = "WF"+dfStr
		DRC_lineToWaterfallVert(waterfallStr,fit2Str)
		DRC_lineToWaterfallVert(waterfallStr,fit1Str)
		DRC_lineToWaterfallBottom(waterfallStr,"edw")
		ModifyGraph rgb=(65280,0,0)
		TextBox/C/N=text1/A=RB msgStr
	endif
end

function DIR_findVertices()
	// data folder and images
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String winStr=SIS_getWinName()
	String stackStr=SIS_getImageStackName()		
	print dfStr,winStr
	// cursors
	variable pA=pcsr(A)
	variable qA=qcsr(A)
	variable pB=pcsr(B)
	variable qB=qcsr(B)
	variable angle=atan((qB-qA)/(pB-pA))*180/pi
	//print angle
	// line through pcsr(A) and pcsr(B)
	variable numer=(qB-qA)-(pB*tan((angle-30)*Pi/180)-pA*tan((angle+30)*Pi/180))
	variable denom= (tan((angle+30)*Pi/180)-tan((angle-30)*Pi/180))
	variable xp=numer/denom
	variable xq=qA+(xp-pA)*tan((angle+30)*Pi/180)
	//print xp,xq
	// put pcsr in centre
	Cursor/I A,$stackStr,xp,xq
end